@@ -24,7 +24,6 @@ import os.path
 import pwd
 import requests
 import re
-import shutil
 import StringIO
 import sys
 from termcolor import colored
@@ -40,17 +39,18 @@ class CLI(object):
         self.cmds.description = "Development environment for big data applications"
         self.cmds.version = ferry.__version__
         self.cmds.usage = "ferry COMMAND [arg...]"
-        self.cmds.add_option("-c", "--conf", "Deployment configuration")
-        self.cmds.add_option("-n", "--dns", "Use custom DNS")
-        self.cmds.add_option("-l", "--log", "Log configuration file")
-        self.cmds.add_option("-k", "--key", "Specify key directory")
-        self.cmds.add_option("-m", "--mode", "Deployment mode")
-        self.cmds.add_option("-u", "--upgrade", "Upgrade Ferry")
         self.cmds.add_option("-b", "--build", "Build Ferry default images")
+        self.cmds.add_option("-c", "--conf", "Deployment configuration")
+        self.cmds.add_option("-d", "--dns", "Specify DNS servers")
+        self.cmds.add_option("-k", "--key", "Specify key directory")
+        self.cmds.add_option("-l", "--log", "Log configuration file")
+        self.cmds.add_option("-m", "--mode", "Deployment mode")
+        self.cmds.add_option("-n", "--naked", "Start in naked mode")
+        self.cmds.add_option("-r", "--retry", "Retry action on failure")
+        self.cmds.add_option("-t", "--net", "Use host network device")
+        self.cmds.add_option("-u", "--upgrade", "Upgrade Ferry")
         self.cmds.add_cmd("build", "Build a Dockerfile")
         self.cmds.add_cmd("clean", "Clean zombie Ferry processes")
-        self.cmds.add_cmd("server", "Start all the servers")
-        self.cmds.add_cmd("deploy", "Deploy a service to the cloud")
         self.cmds.add_cmd("help", "Print this help message")
         self.cmds.add_cmd("info", "Print version information")
         self.cmds.add_cmd("inspect", "Return low-level information on a service")
@@ -62,6 +62,7 @@ class CLI(object):
         self.cmds.add_cmd("pull", "Pull a remote image")
         self.cmds.add_cmd("push", "Push an image to a remote registry")
         self.cmds.add_cmd("rm", "Remove a service or snapshot")
+        self.cmds.add_cmd("server", "Start all the servers")
         self.cmds.add_cmd("snapshot", "Take a snapshot")
         self.cmds.add_cmd("snapshots", "List all snapshots")
         self.cmds.add_cmd("ssh", "Connect to a client/connector")
@@ -135,7 +136,7 @@ class CLI(object):
             image = s[0]
 
         if proto == "image":
-            return self._pull_image(image, registry)
+            return self._pull_image(image)
         else:
             return self._pull_app(image)
 
@@ -394,14 +395,24 @@ class CLI(object):
 
     def _stop_all(self, private_key):
         try:
-            constraints = { 'status' : 'running' }
-            payload = { 'constraints' : json.dumps(constraints) }
-            res = requests.get(self.ferry_server + '/query', params=payload)
-            stacks = json.loads(res.text)
-            for uuid in stacks.keys():
-                self._manage_stacks({'uuid' : uuid,
-                                     'key' : private_key, 
-                                     'action' : 'stop'})
+            # We used to stop all the services when quitting
+            # Ferry, but this does not seem safe since a user
+            # may want to keep running a Hadoop cluster even if
+            # Ferry is shutdown.  
+
+            # constraints = { 'status' : 'running' }
+            # payload = { 'constraints' : json.dumps(constraints) }
+            # res = requests.get(self.ferry_server + '/query', params=payload)
+            # stacks = json.loads(res.text)
+            # for uuid in stacks.keys():
+            #     self._manage_stacks({'uuid' : uuid,
+            #                          'key' : private_key, 
+            #                          'action' : 'stop'})
+
+            # Shutdown any backend services that may have
+            # been started (e.g., OpenStack Heat server, etc.). 
+            res = requests.post(self.ferry_server + '/quit')
+            logging.info(res.text)
         except ConnectionError:
             logging.error("could not connect to ferry server")
         
@@ -494,15 +505,18 @@ class CLI(object):
         except ConnectionError:
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+        except ValueError:
+            json_value = None
 
         connector_ip = None
-        for cg in json_value['connectors']:
-            if not connector_id:
-                connector_ip = cg['entry']['ip']
-                break
-            elif connector_id == cg['uniq']:
-                connector_ip = cg['entry']['ip']
-                break
+        if json_value and 'connectors' in json_value:
+            for cg in json_value['connectors']:
+                if not connector_id and 'ip' in cg['entry']:
+                    connector_ip = cg['entry']['ip']
+                    break
+                elif connector_id == cg['uniq'] and 'ip' in cg['entry']:
+                    connector_ip = cg['entry']['ip']
+                    break
 
         # Now form the ssh command. This just executes in the same shell. 
         if connector_ip:
@@ -516,6 +530,7 @@ class CLI(object):
             os.execv('/usr/bin/ssh', cmd.split())
         else:
             logging.warning("could not find connector %s" % connector_id)
+            return "could not connect to " + str(stack_id)
 
     def _parse_deploy_arg(self, param, args, default):
         pattern = re.compile('--%s=(\w+)' % param)
@@ -525,23 +540,6 @@ class CLI(object):
                 return m.group(1)
         return default
 
-    def _deploy_stack(self, stack_id, args):
-        """
-        Deploy the stack. 
-        """
-        mode = self._parse_deploy_arg('mode', args, default='local')
-        conf = self._parse_deploy_arg('conf', args, default='default')
-
-        payload = { 'uuid' : stack_id,
-                    'mode' : mode,
-                    'conf' : conf }
-        try:
-            res = requests.post(self.ferry_server + '/deploy', data=payload)
-            return res.text
-        except ConnectionError:
-            logging.error("could not connect to ferry server")
-            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
-
     def _manage_stacks(self, stack_info):
         """
         Manage the stack. 
@@ -611,8 +609,9 @@ class CLI(object):
 
     def _format_output(self, reply):
         output = reply['text'] + "\n"
-        for c in reply['msgs'].keys():
-            output += "%s: %s\n" % (c, reply['msgs'][c])
+        if 'msgs' in reply:
+            for c in reply['msgs'].keys():
+                output += "%s: %s\n" % (c, reply['msgs'][c])
         return output
 
     def _start_stack(self, options, args):
@@ -706,9 +705,6 @@ class CLI(object):
             self.installer.stop_web(private_key)
             self.installer._stop_docker_daemon()
             return 'stopped ferry'
-        elif(cmd == 'deploy'):
-            stack_id = args.pop(0)
-            return self._deploy_stack(stack_id, args)
         elif(cmd == 'ls'):
             return self._list_apps()
         elif(cmd == 'info'):
