@@ -16,7 +16,6 @@
 import sys
 import sh
 from string import Template
-from ferry.docker.fabric import DockerFabric
 
 class CassandraClientInitializer(object):
     """
@@ -52,22 +51,29 @@ class CassandraClientInitializer(object):
     def _generate_config_dir(self, uuid):
         return 'cassandra_client' + str(uuid)
 
-    """
-    Get the ports necessary for Gluster. 
-    """
-    def get_necessary_ports(self, num_instances):
+    def get_public_ports(self, num_instances):
+        """
+        Ports to expose to the outside world. 
+        """
         return []
 
-    """
-    Get the internal ports. 
-    """
-    def get_exposed_ports(self, num_instances):
+    def get_internal_ports(self, num_instances):
+        """
+        Ports needed for communication within the network. 
+        This is usually used for internal IPC.
+        """
+        return []
+
+    def get_working_ports(self, num_instances):
+        """
+        Ports necessary to get things working. 
+        """
         return []
 
-    """
-    Get total number of instances.
-    """
     def get_total_instances(self, num_instances, layers):
+        """
+        Get total number of instances.
+        """
         instances = []
 
         for i in range(num_instances):
@@ -125,7 +131,7 @@ class CassandraClientInitializer(object):
     """
     def apply(self, config, containers):
         entry_point = { 'type' : 'cassandra-client' }
-        entry_point['ip'] = containers[0]['data_ip']
+        entry_point['ip'] = containers[0]['manage_ip']
 
         # Get the storage information. 
         storage_entry = self._find_cassandra_storage(containers)
