@@ -9,8 +9,11 @@ def mount(entry_point, mount_point):
     # go ahead and create it. 
     cmd = 'mount -t glusterfs %s %s' % (entry_point,
                                         mount_point)
-    output = Popen(cmd, stdout=PIPE,shell=True).stdout.read()
-    logging.warning(output)
+    proc = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
+    output = proc.stdout.read()
+    err = proc.stderr.read()
+    print output
+    print err
 
 def umount(mount_point):
     cmd = 'cat /etc/mtab | grep /service/data | awk \'{print $2}\''
