@@ -14,6 +14,8 @@
 #
 import datetime
 import grp
+import importlib
+import inspect
 import json
 import logging
 import os
@@ -31,16 +33,13 @@ from sets import Set
 from ferry.install import *
 from ferry.docker.resolve       import DefaultResolver
 from ferry.docker.docker        import DockerInstance
-from ferry.docker.fabric        import DockerFabric
 from ferry.docker.configfactory import ConfigFactory
-from ferry.docker.deploy        import DeployEngine
 
 class DockerManager(object):
-    SSH_PORT = 22
+    SSH_PORT = '22'
 
     def __init__(self):
         # Generate configuration.
-        self.docker = DockerFabric()
         self.config = ConfigFactory()
         self.resolver = DefaultResolver()
 
@@ -76,9 +75,10 @@ class DockerManager(object):
             }
 
         # Initialize the state. 
-        self.deploy = DeployEngine(self.docker)
+        self.docker = self._get_system_backend()
         self._init_state_db()
         self._clean_state_db()
+        logging.warning("using backend %s ver:%s " %(self.docker.name, self.docker.version()))
 
     def _init_state_db(self):
         """
@@ -96,6 +96,30 @@ class DockerManager(object):
         """
         self.cluster_collection.remove( {'status':'removed'} )
 
+    def _load_class(self, class_name):
+        """
+        Dynamically load a class
+        """
+        # Construct the full module path. This lets us filter out only
+        # the deployment engines and ignore all the other imported packages. 
+        s = class_name.split("/")
+        module_path = s[0]
+        clazz_name = s[1]
+        module = importlib.import_module(module_path)
+        for n, o in inspect.getmembers(module):
+            if inspect.isclass(o):
+                if o.__module__ == module_path and o.__name__ == clazz_name:
+                    return o(bootstrap = False)
+        return None
+
+    def _get_system_backend(self):
+        """
+        Return backend information 
+        """
+        config = ferry.install.read_ferry_config()
+        backend = config['system']['backend']
+        return self._load_class(backend)
+
     def _serialize_containers(self, containers):
         info = []
         for c in containers:
@@ -167,7 +191,7 @@ class DockerManager(object):
             service.fabric = self.docker
             return service
         else:
-            logging.error("unknown service " + service_type)
+            logging.error("unknown service " + str(service_type))
             return None
 
     def _get_client_services(self, storage_entry, compute_entry):
@@ -260,20 +284,6 @@ class DockerManager(object):
         else:
             return ""
         
-    def inspect_deployed(self, uuid, registry):
-        """
-        Inspect a deployed stack. 
-        """
-        json_reply = {}
-
-        # Need to inspect the registry to make sure
-        # that all the images are available. 
-
-        return json.dumps(json_reply, 
-                          sort_keys=True,
-                          indent=2,
-                          separators=(',',':'))
-
     def inspect_stack(self, stack_uuid):
         """
         Inspect a running stack. 
@@ -290,7 +300,8 @@ class DockerManager(object):
         storage_uuids = []
         compute_uuids = []
         if cluster and 'backends' in cluster:
-            for b in cluster['backends']['uuids']:
+            backends = cluster['backends']
+            for b in backends['uuids']:
                 if b['storage'] != None:
                     storage_uuids.append(b['storage'])
 
@@ -411,39 +422,31 @@ class DockerManager(object):
             s = self.snapshot_collection.find_one( {'snapshot_uuid':v['snapshot_uuid']} )
             if s:
                 time = v['ts'].strftime("%m/%w/%Y (%I:%M %p)")
+
+            backends = []
+            if 'backends' in v:
+                backends = v['backends']['uuids']
+
+            connectors = []
+            if 'connectors' in v:
+                connectors = v['connectors']
+
             json_reply[v['uuid']] = { 'uuid' : v['uuid'],
                                       'base' : v['base'], 
                                       'ts' : time,
-                                      'backends' : v['backends']['uuids'],
-                                      'connectors': v['connectors'],
+                                      'backends' : backends,
+                                      'connectors': connectors,
                                       'status' : v['status']}
         return json.dumps(json_reply, 
                           sort_keys=True,
                           indent=2,
                           separators=(',',':'))
 
-    def query_deployed(self, conf=None):
+    def quit(self):
         """
-        Query the deployed applications. 
+        Tell the backend services to quit. 
         """
-        json_reply = {}
-
-        cursors = self.deploy.find(conf=conf)
-        for c in cursors:
-            for v in c:
-                time = v['ts'].strftime("%m/%w/%Y (%I:%M %p)")
-                c = self.cluster_collection.find_one( {'uuid':v['cluster_uuid']} )
-                
-                json_reply[v['uuid']] = { 'uuid' : v['uuid'],
-                                          'base' : c['base'], 
-                                          'ts' : time,
-                                          'backends' : c['backends']['uuids'],
-                                          'connectors': c['connectors'],
-                                          'status': 'deployed' }
-        return json.dumps(json_reply, 
-                          sort_keys=True,
-                          indent=2,
-                          separators=(',',':'))
+        self.docker.quit()
 
     def _new_service_uuid(self):
         """
@@ -497,18 +500,6 @@ class DockerManager(object):
     def is_removed(self, uuid, conf=None):
         return self._confirm_status(uuid, 'removed')
     
-    def is_deployed(self, uuid, conf=None):
-        """
-        Check if the UUID is of a deployed application. 
-        """
-        v = self.deploy.find( one=True, 
-                              spec = {'uuid':uuid},
-                              conf = conf )
-        if v:
-            return True
-        else:
-            return False
-
     def is_installed(self, app):
         """
         Indicate whether the application is installed. 
@@ -516,20 +507,20 @@ class DockerManager(object):
         apps = self.query_applications()
         return app in apps
 
-    def get_base_image(self, uuid):
+    def get_stack(self, uuid):
         """
         Get the base image of this cluster. 
         """
         cluster = self.cluster_collection.find_one( {'uuid':uuid} )
         if cluster:
-            return cluster['base']
+            return cluster
         return None
 
     def _new_data_dir(self, service_uuid, storage_type, storage_id):
         """
         Create a new data directory
         """
-        scratch_dir = ferry.install._get_ferry_scratch()
+        scratch_dir = self.docker.get_data_dir()
 
         # First check if this data directory already exists. If so,
         # go ahead and delete it (this will hopefully get rid of all xattr stuff)
@@ -540,7 +531,7 @@ class DockerManager(object):
         """
         Create a new log directory
         """
-        scratch_dir = ferry.install._get_ferry_scratch()
+        scratch_dir = self.docker.get_data_dir()
 
         # First check if this data directory already exists. If so,
         # go ahead and delete it (this will hopefully get rid of all xattr stuff)
@@ -576,13 +567,14 @@ class DockerManager(object):
         container_dir = service.container_data_dir
         log_dir = service.container_log_dir
         host_name = service.new_host_name(instance)
-        ports = service.get_necessary_ports(num_instances)
-        exposed = service.get_exposed_ports(num_instances)
+        ports = service.get_public_ports(num_instances)
+        internal = service.get_internal_ports(num_instances)
+        exposed = service.get_working_ports(num_instances)
         
         # Add SSH port for management purposes. 
         exposed.append(DockerManager.SSH_PORT)
 
-        return container_dir, log_dir, host_name, ports, exposed
+        return container_dir, log_dir, host_name, ports, exposed, internal
 
     def _read_key_dir(self, private_key):
         """
@@ -621,7 +613,7 @@ class DockerManager(object):
         for t in instances:
             instance_type = self._get_instance_image(t)
             service = self._get_service(t)
-            container_dir, log_dir, host_name, ports, exposed = self._get_service_environment(service, i, num_instances)
+            container_dir, log_dir, host_name, ports, exposed, internal = self._get_service_environment(service, i, num_instances)
             new_log_dir = self._new_log_dir(service_uuid, t, i, replace=replace)
             dir_info = { new_log_dir : log_dir }
 
@@ -640,6 +632,7 @@ class DockerManager(object):
                               'privatekey': key_name, 
                               'ports':ports,
                               'exposed':exposed, 
+                              'internal':internal,
                               'hostname':host_name,
                               'args':args}
             plan['localhost']['containers'].append(container_info)
@@ -672,7 +665,7 @@ class DockerManager(object):
         for t in instances:
             instance_type = self._get_instance_image(t)
             service = self._get_service(t)
-            container_dir, log_dir, host_name, ports, exposed = self._get_service_environment(service, i, num_instances)
+            container_dir, log_dir, host_name, ports, exposed, internal = self._get_service_environment(service, i, num_instances)
             new_log_dir = self._new_log_dir(service_uuid, t, i)
             dir_info = { new_log_dir : log_dir }
             container_info = {'image':instance_type,
@@ -684,6 +677,7 @@ class DockerManager(object):
                               'type':t, 
                               'ports':ports,
                               'exposed':exposed, 
+                              'internal':internal,
                               'hostname':host_name,
                               'args':args}
             plan['localhost']['containers'].append(container_info)
@@ -729,6 +723,7 @@ class DockerManager(object):
                            'type':connector_type, 
                            'ports':ports,
                            'exposed':[], 
+                           'internal':[],
                            'hostname':host_name,
                            'name':name, 
                            'args':args}
@@ -752,11 +747,15 @@ class DockerManager(object):
         Transfer the hostname/IP addresses to all the containers. 
         """
         with open('/tmp/instances', 'w+') as hosts_file:
+            # Each line has the form (private IP, public IP, hostname)
+            # We want to use the private IP for the hosts file. 
             for ip in ips:
-                hosts_file.write("%s %s\n" % (ip[0], ip[1]))
+                hosts_file.write("%s %s\n" % (ip[0], ip[2]))
         for ip in ips:
-            self.docker.copy_raw(private_key, ip[0], '/tmp/instances', '/service/sconf/instances')
-            self.docker.cmd_raw(private_key, ip[0], '/service/sbin/startnode hosts')
+            # However, we want to use the public IP for actually copying
+            # the hosts data. 
+            self.docker.copy_raw(private_key, ip[1], '/tmp/instances', '/service/sconf/instances', self.docker.docker_user)
+            self.docker.cmd_raw(private_key, ip[1], '/service/sbin/startnode hosts', self.docker.docker_user)
         
     def _transfer_env_vars(self, containers, env_vars):
         """
@@ -768,67 +767,75 @@ class DockerManager(object):
             self.docker.cmd(containers, 
                             "echo export %s=%s >> /etc/profile" % (k, env_vars[k]))
 
-    def _start_containers(self, plan):
+    def _start_containers(self, cluster_uuid, service_uuid, plan, ctype):
         """
         Start the containers on the specified environment
         """
-        return self.docker.alloc(plan['localhost']['containers']);
+        return self.docker.alloc(cluster_uuid = cluster_uuid,
+                                 service_uuid = service_uuid, 
+                                 container_info = plan['localhost']['containers'], 
+                                 ctype = ctype);
 
-    def _restart_containers(self, containers):
+    def _restart_containers(self, cluster_uuid, service_uuid, containers):
         """
         Restart the stopped containers. 
         """
-        return self.docker.restart(containers)
+        return self.docker.restart(cluster_uuid, service_uuid, containers)
 
-    def cancel_stack(self, backends, connectors):
+    def cancel_stack(self, cluster_uuid, backends, connectors):
         """
         The stack could not be instantiated correctly. Just get rid
         of these containers. 
         """
         for b in backends['uuids']:
             conf = self._get_service_configuration(b['storage'], detailed=True)
+            
             if conf and 'containers' in conf:
-                for c in conf['containers']:
-                    self.docker.stop([c])
+                self.docker.stop(cluster_uuid, b['storage'], conf['containers'])
+            else:
+                self.docker.stop(cluster_uuid, b['storage'], [])
+
             for compute in b['compute']:
                 conf = self._get_service_configuration(compute, detailed=True)
                 if conf and 'containers' in conf:
-                    for c in conf['containers']:
-                        self.docker.stop([c])
+                    self.docker.stop(cluster_uuid, compute, conf['containers'])
+                else:
+                    self.docker.stop(cluster_uuid, compute, [])
 
         for b in connectors:
             s = self._get_service_configuration(b, detailed=True)
             if s and 'containers' in s:
-                for c in s['containers']:
-                    self.docker.stop([c])
+                self.docker.stop(cluster_uuid, b, s['containers'])
+            else:
+                self.docker.stop(cluster_uuid, b, [])
 
-    def register_stack(self, backends, connectors, base, key=None, uuid=None):
+    def reserve_stack(self):
+        """
+        Reserve a UUID. 
+        """
+        return self._new_stack_uuid()
+
+    def register_stack(self, backends, connectors, base, cluster_uuid, status, key=None, new_stack=True):
         """
         Register the set of services under a single cluster identifier. 
         """
-        if not uuid:
-            cluster_uuid = self._new_stack_uuid()
-        else:
-            cluster_uuid = uuid
-
         ts = datetime.datetime.now()
         cluster = { 'uuid' : cluster_uuid,
-                    'backends':backends,
-                    'connectors':connectors,
                     'num_snapshots':0,
                     'snapshot_ts':'', 
                     'snapshot_uuid':base, 
                     'base':base,
-                    'status': 'running',
+                    'backends' : backends,
+                    'connectors' : connectors, 
+                    'status': status,
+                    'key' : key, 
                     'ts':ts }
 
-        if not uuid:
-            cluster['key'] = key
+        if new_stack:
             self.cluster_collection.insert( cluster )
         else:
-            self._update_stack(uuid, cluster)
-
-        return cluster_uuid
+            self.cluster_collection.remove( {'uuid' : cluster_uuid} )
+            self.cluster_collection.insert( cluster )
 
     def _update_stack(self, cluster_uuid, state):
         """
@@ -847,7 +854,8 @@ class DockerManager(object):
             connector_uuids = cluster['connectors']
             for c in connector_uuids:
                 connectors = {'uuid' : c,
-                              'instances' : []}
+                              'instances' : [],
+                              'type' : None }
                 connector_info = self._get_service_configuration(c, detailed=True)
                 if connector_info:
                     for connector in connector_info['containers']:
@@ -861,7 +869,8 @@ class DockerManager(object):
             for b in backends['uuids']:
                 if b['storage'] != None:
                     storage = {'uuid' : b['storage'],
-                               'instances' : []}
+                               'instances' : [],
+                               'type' : None }
                     storage_info = self._get_service_configuration(b['storage'], detailed=True)
                     if storage_info:
                         for s in storage_info['containers']:
@@ -873,7 +882,8 @@ class DockerManager(object):
                 if b['compute'] != None:
                     for c in b['compute']:
                         compute = {'uuid' : c,
-                                   'instances' : []}
+                                   'instances' : [],
+                                   'type' : None}
                         compute_info = self._get_service_configuration(c, detailed=True)
                         if compute_info:
                             for container in compute_info['containers']:
@@ -898,24 +908,24 @@ class DockerManager(object):
 
         # Then actually stop the containers. 
         for c in connectors:
-            self.docker.halt(c['instances'])
+            self.docker.halt(cluster_uuid, c['uuid'], c['instances'])
         for c in compute:
-            self.docker.halt(c['instances'])
+            self.docker.halt(cluster_uuid, c['uuid'], c['instances'])
         for c in storage:
-            self.docker.halt(c['instances'])
+            self.docker.halt(cluster_uuid, c['uuid'], c['instances'])
 
     def _purge_stack(self, cluster_uuid):
         volumes = []
         connectors, compute, storage = self._get_cluster_instances(cluster_uuid)
         for c in connectors:
-            self.docker.remove(c['instances'])
+            self.docker.remove(cluster_uuid, c['uuid'], c['instances'])
         for c in compute:
-            self.docker.remove(c['instances'])
+            self.docker.remove(cluster_uuid, c['uuid'], c['instances'])
         for s in storage:
             for i in s['instances']:
                 for v in i.volumes.keys():
                     volumes.append(v)
-            self.docker.remove(s['instances'])
+            self.docker.remove(cluster_uuid, s['uuid'], s['instances'])
 
         # Now remove the data directories. 
         for v in volumes:
@@ -942,7 +952,7 @@ class DockerManager(object):
 
             # Register the snapshot in the snapshot state. 
             snapshot_uuid = self._new_snapshot_uuid(cluster_uuid)
-            snapshot_ts = datetime.datetime.now()
+            snapshot_ts = str(datetime.datetime.now())
             snapshot_state = { 'snapshot_ts' : snapshot_ts, 
                                'snapshot_uuid' : snapshot_uuid,
                                'snapshot_cs' : cs_snapshots,
@@ -959,10 +969,14 @@ class DockerManager(object):
         """
         Start the service.
         """
-        service_info = self._get_service_configuration(uuid, detailed=True)
-        return self._start_service(uuid, containers, service_info)
+        if len(containers) > 0:
+            service_info = self._get_service_configuration(uuid, detailed=True)
+            return self._start_service(uuid, containers, service_info)
+        else:
+            return {}
 
     def allocate_compute(self,
+                         cluster_uuid, 
                          compute_type, 
                          key_name, 
                          storage_uuid,
@@ -989,17 +1003,27 @@ class DockerManager(object):
         storage_entry = self._get_service_configuration(storage_uuid)
 
         # Allocate all the containers. 
-        containers = self._start_containers(plan)
-
-        # Generate a configuration dir.
-        config_dirs, entry_point = self.config.generate_compute_configuration(service_uuid, 
-                                                                              containers, 
-                                                                              service, 
-                                                                              args, 
-                                                                              [storage_entry])
-
-        # Now copy over the configuration.
-        self._transfer_config(config_dirs)
+        containers = self._start_containers(cluster_uuid, service_uuid, plan, ctype="compute")
+        if containers == None:
+            # The containers could not be allocated for some
+            # reason. Make sure the calling method knows this so
+            # that it can return a failure message.
+            return None, None
+        elif len(containers) > 0:
+            # The containers were instantiated successfully. 
+            # Generate storage-specific configuration and transfer
+            # it over to the new containers. 
+            config_dirs, entry_point = self.config.generate_compute_configuration(service_uuid, 
+                                                                                  containers, 
+                                                                                  service, 
+                                                                                  args, 
+                                                                                  [storage_entry])
+            self._transfer_config(config_dirs)
+        else:
+            # The container allocator did not allocate any containers
+            # but it did so without any errors (perhaps the user requested
+            # zero containers?). 
+            entry_point = {}
 
         # Update the service configuration. 
         container_info = self._serialize_containers(containers)
@@ -1035,43 +1059,49 @@ class DockerManager(object):
                          uuid,
                          containers,
                          service_type):
-        service_info = self._get_service_configuration(uuid, detailed=True)
-        entry_point = service_info['entry']        
-        if service_info['class'] != 'connector':
-            service = self._get_service(service_type)
-            return service.restart_service(containers, entry_point, self.docker)
+        if len(containers) > 0:
+            service_info = self._get_service_configuration(uuid, detailed=True)
+            entry_point = service_info['entry']        
+            if service_info['class'] != 'connector':
+                service = self._get_service(service_type)
+                return service.restart_service(containers, entry_point, self.docker)
+            else:
+                all_output = {}
+                backends = service_info['backends']
+                for backend in backends:
+                    if 'client' in self.service[backend]:
+                        service = self.service[backend]['client']
+                        output = service.restart_service(containers, entry_point, self.docker)
+                        if output:
+                            all_output = dict(all_output.items() + output.items())
+                return all_output
         else:
-            all_output = {}
-            for backend in service_info['backends']:
-                if 'client' in self.service[backend]:
-                    service = self.service[backend]['client']
-                    output = service.restart_service(containers, entry_point, self.docker)
-                    if output:
-                        all_output = dict(all_output.items() + output.items())
-            return all_output
+            return {}
 
     def _stop_service(self,
                       uuid,
                       containers,
                       service_type):
-        service_info = self._get_service_configuration(uuid, detailed=True)
-        entry_point = service_info['entry']        
-        if service_info['class'] != 'connector':
-            service = self._get_service(service_type)
-            service.stop_service(containers, entry_point, self.docker)
-        else:
-            for backend in service_info['backends']:
-                if 'client' in self.service[backend]:
-                    service = self.service[backend]['client']
-                    service.stop_service(containers, entry_point, self.docker)
+        if len(containers) > 0:
+            service_info = self._get_service_configuration(uuid, detailed=True)
+            entry_point = service_info['entry']        
+            if service_info['class'] != 'connector':
+                service = self._get_service(service_type)
+                service.stop_service(containers, entry_point, self.docker)
+            else:
+                backends = service_info['backends']
+                for backend in backends:
+                    if 'client' in self.service[backend]:
+                        service = self.service[backend]['client']
+                        service.stop_service(containers, entry_point, self.docker)
         
-    def restart_containers(self, service_uuid, containers):
+    def restart_containers(self, cluster_uuid, service_uuid, containers):
         """
         Restart an stopped storage cluster. This does not
         re-initialize the container. It just starts an empty
         container. 
         """
-        self._restart_containers(containers)
+        self._restart_containers(cluster_uuid, service_uuid, containers)
         container_info = self._serialize_containers(containers)
 
         service = {'uuid':service_uuid, 
@@ -1080,6 +1110,7 @@ class DockerManager(object):
         self._update_service_configuration(service_uuid, service)
                         
     def allocate_storage(self, 
+                         cluster_uuid, 
                          storage_type, 
                          key_name, 
                          num_instances=1,
@@ -1105,16 +1136,26 @@ class DockerManager(object):
                                                  replace = replace)
 
         # Allocate all the containers. 
-        containers = self._start_containers(plan)
-
-        # Generate a configuration dir.
-        config_dirs, entry_point = self.config.generate_storage_configuration(service_uuid, 
-                                                                              containers, 
-                                                                              service, 
-                                                                              args)
-
-        # Now copy over the configuration.
-        self._transfer_config(config_dirs)
+        containers = self._start_containers(cluster_uuid, service_uuid, plan, ctype="storage")
+        if containers == None:
+            # The containers could not be allocated for some
+            # reason. Make sure the calling method knows this so
+            # that it can return a failure message.
+            return None, None
+        elif len(containers) > 0:
+            # The containers were instantiated successfully. 
+            # Generate storage-specific configuration and transfer
+            # it over to the new containers. 
+            config_dirs, entry_point = self.config.generate_storage_configuration(service_uuid, 
+                                                                                  containers, 
+                                                                                  service, 
+                                                                                  args)
+            self._transfer_config(config_dirs)
+        else:
+            # The container allocator did not allocate any containers
+            # but it did so without any errors (perhaps the user requested
+            # zero containers?). 
+            entry_point = {}
 
         container_info = self._serialize_containers(containers)
         service = {'uuid':service_uuid, 
@@ -1132,46 +1173,6 @@ class DockerManager(object):
         """
         return FERRY_HOME + '/data/conf/deploy_default.json'
 
-    def _get_deploy_params(self, mode, conf):
-        """
-        Get the deployment configuration parameters. 
-        """
-        # First just find and read the configuration file. 
-        if conf == 'default':
-            conf = self._get_default_conf()
-
-        # Read the configuration file.
-        if os.path.exists(conf):
-            with open(conf, 'r') as f: 
-                j = json.loads(f.read())
-
-                # Now find the right configuration.
-                if mode in j:
-                    j[mode]['_mode'] = mode
-                    return j[mode]
-        return None
-
-    def deploy_stack(self, cluster_uuid, params=None):
-        """
-        Deploy an existing stack. 
-        """
-        containers = []
-        cluster = self.cluster_collection.find_one( {'uuid':cluster_uuid} )
-        if cluster:
-            connector_uuids = cluster['connectors']
-            for c in connector_uuids:
-                connector_info = self._get_service_configuration(c, detailed=True)
-                if connector_info:
-                    containers.append(DockerInstance(connector_info['containers'][0]))
-
-            self.deploy.deploy(cluster_uuid, containers, params)
-
-            # Check if we need to try starting the
-            # stack right away. 
-            if params and 'start-on-create' in params:
-                return True
-        return False
-
     def manage_stack(self,
                      stack_uuid,
                      private_key, 
@@ -1218,7 +1219,6 @@ class DockerManager(object):
                 storage_uuid = uuid['storage']
                 storage_conf = self._get_service_configuration(storage_uuid, 
                                                                detailed=True)
-
                 compute_confs = []
                 if 'compute' in uuid:
                     for c in uuid['compute']:
@@ -1238,18 +1238,7 @@ class DockerManager(object):
         """
         snapshot = self.cluster_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
         if snapshot:
-            return snapshot['backends']['backend']
-
-    def fetch_deployed_backend(self, app_uuid, conf=None):
-        """
-        Lookup the deployed backend info. 
-        """
-        app = self.deploy.find( one = True,
-                                spec = { 'uuid' : app_uuid },
-                                conf = conf )
-        stack = self.cluster_collection.find_one( {'uuid':app['cluster_uuid'] } )
-        if stack:
-            return stack['backends']['backend']
+            return snapshot['backends']
 
     def allocate_stopped_connectors(self, 
                                      app_uuid, 
@@ -1262,7 +1251,8 @@ class DockerManager(object):
         connector_plan = []
         cluster = self.cluster_collection.find_one( {'uuid':app_uuid} )
         if cluster:
-            for cuid in cluster['connectors']:
+            connectors = cluster['connectors']
+            for cuid in connectors:
                 # Retrieve the actual service information. This will
                 # contain the container ID. 
                 s = self._get_service_configuration(cuid, detailed=True)
@@ -1274,39 +1264,11 @@ class DockerManager(object):
                                              'backend' : backend_info, 
                                              'start' : 'restart' } )
                     connector_info.append(cuid)
-                    self.restart_containers(cuid, containers)
+                    self.restart_containers(app_uuid, cuid, containers)
         return connector_info, connector_plan
 
-    def allocate_deployed_connectors(self, 
-                                     app_uuid, 
-                                     key_name, 
-                                     backend_info,
-                                     conf = None):
-        """
-        Lookup the deployed application connector info and instantiate. 
-        """
-        connector_info = []
-        connector_plan = []
-        app = self.deploy.find( one = True,
-                                spec = { 'uuid' : app_uuid },
-                                conf = conf)
-        if app:
-            for c in app['connectors']:
-                uuid, containers = self.allocate_connector(connector_type = c['type'],
-                                                           key_name = key_name,  
-                                                           backend = backend_info,
-                                                           name = c['name'], 
-                                                           args = c['args'],
-                                                           ports = c['ports'].keys(),
-                                                           image = c['image'])
-                connector_info.append(uuid)
-                connector_plan.append( { 'uuid' : uuid,
-                                         'containers' : containers,
-                                         'type' : c['type'], 
-                                         'start' : 'start' } )
-        return connector_info, connector_plan
-                
     def allocate_snapshot_connectors(self, 
+                                     cluster_uuid, 
                                      snapshot_uuid, 
                                      key_name, 
                                      backend_info):
@@ -1318,7 +1280,8 @@ class DockerManager(object):
         snapshot = self.snapshot_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
         if snapshot:
             for s in snapshot['snapshot_cs']:
-                uuid, containers = self.allocate_connector(connector_type = s['type'],
+                uuid, containers = self.allocate_connector(cluster_uuid = cluster_uuid,
+                                                           connector_type = s['type'],
                                                            key_name = key_name, 
                                                            backend = backend_info,
                                                            name = s['name'], 
@@ -1371,7 +1334,7 @@ class DockerManager(object):
             self._transfer_env_vars(connectors, env_vars)
 
         # Start the containers and update the state. 
-        self._restart_service(service_uuid, connectors, connectors[0].service_type)
+        output = self._restart_service(service_uuid, connectors, connectors[0].service_type)
         container_info = self._serialize_containers(connectors)
         service = {'uuid':service_uuid, 
                    'containers':container_info, 
@@ -1379,8 +1342,10 @@ class DockerManager(object):
                    'entry':entry_points,
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
-                
+        return output
+
     def allocate_connector(self,
+                           cluster_uuid, 
                            connector_type, 
                            key_name, 
                            backend=None,
@@ -1416,25 +1381,38 @@ class DockerManager(object):
                                                    name = name,
                                                    ports = ports, 
                                                    args = args)
-        containers = self._start_containers(plan)
+        containers = self._start_containers(cluster_uuid, service_uuid, plan, ctype="connector")
+        if containers == None:
+            # The containers could not be allocated for some
+            # reason. Make sure the calling method knows this so
+            # that it can return a failure message.
+            return None, None
+        elif len(containers) > 0:            
+            # The containers were instantiated successfully. 
+            # Generate storage-specific configuration and transfer
+            # it over to the new containers.         
+            entry_points = {}
+            backend_names = []
+            services, backend_names = self._get_client_services(storage_entry, compute_entry)
+            for service in services:
+                config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
+                                                                                        containers, 
+                                                                                        service,
+                                                                                        storage_entry,
+                                                                                        compute_entry,
+                                                                                        args)
+                # Merge all the entry points. 
+                entry_points = dict(entry_point.items() + entry_points.items())
 
-        # Now generate the configuration files that will be
-        # transferred to the containers. 
-        entry_points = {}
-        services, backend_names = self._get_client_services(storage_entry, compute_entry)
-        for service in services:
-            config_dirs, entry_point = self.config.generate_connector_configuration(service_uuid, 
-                                                                                    containers, 
-                                                                                    service,
-                                                                                    storage_entry,
-                                                                                    compute_entry,
-                                                                                    args)
-            # Merge all the entry points. 
-            entry_points = dict(entry_point.items() + entry_points.items())
-
-            # Now copy over the configuration.
-            self._transfer_config(config_dirs)
-            self._transfer_env_vars(containers, env_vars)
+                # Now copy over the configuration.
+                self._transfer_config(config_dirs)
+                self._transfer_env_vars(containers, env_vars)
+        else:
+            # The container allocator did not allocate any containers
+            # but it did so without any errors (perhaps the user requested
+            # zero containers?). 
+            entry_points = {}
+            backend_names = []
 
         # Update the connector state. 
         container_info = self._serialize_containers(containers)
