@@ -34,7 +34,7 @@ import yaml
 from distutils import spawn
 from ferry.ip.client import DHCPClient
 from ferry.config.mongo.mongoconfig import *
-from ferry.docker.fabric import DockerFabric
+from ferry.fabric.local import LocalFabric
 from string import Template
 from subprocess import Popen, PIPE
 
@@ -50,6 +50,12 @@ def _get_ferry_home():
     else:
         return os.path.dirname(__file__)
 
+def _get_conf_dir():
+    if 'HOME' in os.environ:
+        return os.environ['HOME']
+    else:
+        return "/root"
+
 def _get_docker_registry():
     if 'DOCKER_REGISTRY' in os.environ:
         return os.environ['DOCKER_REGISTRY']
@@ -72,17 +78,6 @@ def _get_ferry_dir(server):
         else:
             return os.environ['HOME'] + '/.ferry'
 
-def _get_ferry_scratch():
-    if 'FERRY_SCRATCH' in os.environ:
-        scratch_dir = os.environ['FERRY_SCRATCH']
-    else:
-        scratch_dir = os.path.join(_get_ferry_dir(server=True), 'scratch')
-
-    if not os.path.isdir(scratch_dir):
-        os.makedirs(scratch_dir)
-
-    return scratch_dir
-
 def _get_ferry_user():
     uid = pwd.getpwnam("root").pw_uid
     gid = grp.getgrnam("docker").gr_gid
@@ -133,6 +128,9 @@ def _touch_file(file_name, content, root=False):
     # "open" will throw an exception. 
     if not os.path.isdir(os.path.dirname(file_name)):
         os.makedirs(os.path.dirname(file_name))
+        if root:
+            uid, gid = _get_ferry_user()
+            os.chown(os.path.dirname(file_name), uid, gid)
 
     try:
         f = open(file_name, 'w+')
@@ -150,7 +148,6 @@ FERRY_HOME=_get_ferry_home()
 DEFAULT_IMAGE_DIR=FERRY_HOME + '/data/dockerfiles'
 DEFAULT_KEY_DIR=FERRY_HOME + '/data/key'
 DEFAULT_SSH_KEY=DEFAULT_KEY_DIR + '/insecure_ferry_key.pem'
-DEFAULT_DOCKER_LOGIN=os.environ['HOME'] + '/.ferry.docker'
 DEFAULT_DOCKER_REPO='ferry'
 GUEST_DOCKER_REPO='ferry-user'
 DEFAULT_FERRY_OWNER='ferry:docker'
@@ -159,6 +156,8 @@ DOCKER_CMD='docker-ferry'
 DOCKER_SOCK='unix:////var/run/ferry.sock'
 DOCKER_PID='/var/run/ferry.pid'
 DOCKER_DIR=_get_ferry_dir(server=True)
+USER_FERRY_CONFIG=os.environ['HOME'] + '/.ferry-config.yaml'
+SYSTEM_FERRY_CONFIG=FERRY_HOME + '/data/conf/ferry-default.yaml'
 DEFAULT_TEMPLATE_DIR=FERRY_HOME + '/data/templates'
 DEFAULT_BUILTIN_APPS=FERRY_HOME + '/data/plans'
 DEFAULT_FERRY_APPS=DOCKER_DIR + '/apps'
@@ -167,25 +166,53 @@ DEFAULT_MONGO_LOG=DOCKER_DIR + '/mongolog'
 DEFAULT_REGISTRY_DB=DOCKER_DIR + '/registry'
 DEFAULT_DOCKER_LOG=DOCKER_DIR + '/docker.log'
 
+def _recursive_merge(dict1, dict2):
+    """
+    Recursively merge the content of dict2 into dict1.
+    dict2 will always have precedence. 
+    """
+
+    for k, v in dict2.iteritems():
+        if type(v) is dict:
+            if not k in dict1:
+                dict1[k] = v
+            else:
+                _recursive_merge(dict1[k], v)
+        else:
+            dict1[k] = v
+
+def read_ferry_config():
+    """
+    Read and store the Ferry user configuration. 
+    """
+    system = {}
+    user = {}
+        
+    if os.path.exists(ferry.install.SYSTEM_FERRY_CONFIG):
+        f = open(ferry.install.SYSTEM_FERRY_CONFIG, 'r')
+        system = yaml.load(f)
+
+    if os.path.exists(ferry.install.USER_FERRY_CONFIG):
+        f = open(ferry.install.USER_FERRY_CONFIG, 'r') 
+        user = yaml.load(f)
+
+    _recursive_merge(system, user)
+    return system
+
 class Installer(object):
     def __init__(self, cli=None):
         self.network = DHCPClient()
-        self.fabric = DockerFabric(bootstrap=True)
+        self.fabric = LocalFabric(bootstrap=True)
         self.mongo = MongoInitializer()
         self.mongo.fabric = self.fabric
         self.mongo.template_dir = DEFAULT_TEMPLATE_DIR + '/mongo/'
         self.cli = cli
+        self.config = read_ferry_config()
 
     def get_ferry_account(self):
-        """
-        Read in the remote Ferry DB account information. Used
-        for registering applications. 
-        """
-        with open(ferry.install.DEFAULT_DOCKER_LOGIN, 'r') as f:
-            args = yaml.load(f)
-            args = args['ferry']
-            if all(k in args for k in ("user","key","server")):
-                return args['user'], args['key'], args['server']
+        args = self.config['ferry']
+        if all(k in args for k in ("user","key","server")):
+            return args['user'], args['key'], args['server']
         return None, None, None
 
     def create_signature(self, request, key):
@@ -220,6 +247,46 @@ class Installer(object):
         """
         self.network.clean_rules()
 
+    def _install_perform_build(self, options, num_tries):
+        # Normally we don't want to build the Dockerfiles,
+        # but sometimes we may for testing, etc. 
+        build = False
+        if options and '-b' in options:
+            build = True
+
+        if options and '-u' in options:
+            if len(options['-u']) > 0 and options['-u'][0] != True:
+                logging.warning("performing select rebuild (%s)" % str(options['-u']))
+                self.build_from_list(options['-u'], 
+                                     DEFAULT_IMAGE_DIR,
+                                     DEFAULT_DOCKER_REPO, build, recurse=False)
+            else:
+                logging.warning("performing forced rebuild")
+                self.build_from_dir(DEFAULT_IMAGE_DIR, DEFAULT_DOCKER_REPO, build)
+        else:
+            # We want to be selective about which images
+            # to rebuild. Useful if one image breaks, etc. 
+            to_build = self.check_images(DEFAULT_IMAGE_DIR,
+                                         DEFAULT_DOCKER_REPO)
+            if len(to_build) > 0:
+                logging.warning("performing select rebuild (%s)" % str(to_build))
+                self.build_from_list(to_build, 
+                                     DEFAULT_IMAGE_DIR,
+                                     DEFAULT_DOCKER_REPO, build)
+
+        # Check that all the images were built.
+        not_installed = self._check_all_images()
+        if len(not_installed) == 0:
+            return 'installed ferry'
+        else:
+            logging.error('images not built: ' + str(not_installed))
+            if num_tries == 0:
+                return 'Some images were not installed. Please type \'ferry install\' again.'
+            else:
+                # Try building the images again.
+                logging.info("retrying install (%d)" % num_tries)
+                return self._install_perform_build(options, num_tries - 1)
+
     def install(self, args, options):
         # Check if the host is actually 64-bit. If not raise a warning and quit.
         if not _supported_arch():
@@ -256,39 +323,14 @@ class Installer(object):
             logging.error('ferry docker daemon not started')
             return msg
 
-        # Normally we don't want to build the Dockerfiles,
-        # but sometimes we may for testing, etc. 
-        build = False
-        if options and '-b' in options:
-            build = True
-
-        if options and '-u' in options:
-            if len(options['-u']) > 0 and options['-u'][0] != True:
-                logging.warning("performing select rebuild (%s)" % str(options['-u']))
-                self.build_from_list(options['-u'], 
-                                     DEFAULT_IMAGE_DIR,
-                                     DEFAULT_DOCKER_REPO, build, recurse=False)
-            else:
-                logging.warning("performing forced rebuild")
-                self.build_from_dir(DEFAULT_IMAGE_DIR, DEFAULT_DOCKER_REPO, build)
+        # Perform the actual build/download. Sometimes the download
+        # may fail so give the user a chance to automatically retry
+        # a few times before giving up. 
+        if options and '-r' in options:
+            num_tries = int(options['-r'])
         else:
-            # We want to be selective about which images
-            # to rebuild. Useful if one image breaks, etc. 
-            to_build = self.check_images(DEFAULT_IMAGE_DIR,
-                                         DEFAULT_DOCKER_REPO)
-            if len(to_build) > 0:
-                logging.warning("performing select rebuild (%s)" % str(to_build))
-                self.build_from_list(to_build, 
-                                     DEFAULT_IMAGE_DIR,
-                                     DEFAULT_DOCKER_REPO, build)
-
-        # Check that all the images were built.
-        not_installed = self._check_all_images()
-        if len(not_installed) == 0:
-            return 'installed ferry'
-        else:
-            logging.error('images not built: ' + str(not_installed))
-            return 'Some images were not installed. Please type \'ferry install\' again.'
+            num_tries = 0
+        return self._install_perform_build(options, num_tries)
 
     def _check_all_images(self):
         not_installed = []
@@ -345,6 +387,11 @@ class Installer(object):
         # Check if the ssh key permission is properly set. 
         self._check_and_change_ssh_keyperm()
 
+        # Check if we're operating in naked mode. If so, 
+        # we just needed to start the docker daemon, so we're all done!
+        if options and '-n' in options:
+            return
+
         # Check if the user-application directory exists.
         # If not, create it. 
         try:
@@ -392,20 +439,26 @@ class Installer(object):
                      'volumes':volumes,
                      'volume_user':DEFAULT_FERRY_OWNER, 
                      'ports':[],
-                     'exposed':self.mongo.get_exposed_ports(1), 
+                     'exposed':self.mongo.get_working_ports(1), 
+                     'internal':self.mongo.get_internal_ports(1),
                      'hostname':'ferrydb',
                      'netenable':True, 
                      'args': 'trust'
                      }
         mongoconf = self.mongo.generate(1)
         mongoconf.uuid = 'fdb-' + str(uuid.uuid4()).split('-')[0]
-        mongobox = self.fabric.alloc([mongoplan])[0]
-        if not mongobox:
+        containers = self.fabric.alloc(mongoconf.uuid,
+                                     mongoconf.uuid,
+                                     [mongoplan], 
+                                     "MONGO")
+        if containers and len(containers) > 0:
+            mongobox = containers[0]
+        else:
             logging.error("Could not start MongoDB image")
             sys.exit(1)
 
         ip = mongobox.internal_ip
-        _touch_file('/tmp/mongodb.ip', ip, root=True)
+        _touch_file('/tmp/ferry/mongodb.ip', ip, root=True)
 
         # Once the container is started, we'll need to copy over the
         # configuration files, and then manually send the 'start' command. 
@@ -436,9 +489,11 @@ class Installer(object):
         # Reserve the Mongo IP.
         self.network.reserve_ip(ip)
 
-        # Start the Ferry HTTP servers
+        # Start the Ferry HTTP server. We used to start multiple
+        # workers, but since we operate in an asynch. manner, this
+        # shouldn't be necessary anymore. 
         logging.warning("starting http servers on port 4000 and mongo %s" % ip)
-        cmd = 'gunicorn -e FERRY_HOME=%s -t 3600 -w 3 -b 127.0.0.1:4000 ferry.http.httpapi:app &' % FERRY_HOME
+        cmd = 'gunicorn -e FERRY_HOME=%s -t 3600 -w 1 -b 127.0.0.1:4000 ferry.http.httpapi:app &' % FERRY_HOME
         Popen(cmd, stdout=PIPE, shell=True, env=my_env)
 
     def _force_stop_web(self):
@@ -448,8 +503,8 @@ class Installer(object):
 
     def stop_web(self, key):
         # Shutdown the mongo instance
-        if os.path.exists('/tmp/mongodb.ip'):
-            f = open('/tmp/mongodb.ip', 'r')
+        if os.path.exists('/tmp/ferry/mongodb.ip'):
+            f = open('/tmp/ferry/mongodb.ip', 'r')
             ip = f.read().strip()
             f.close()
 
@@ -461,7 +516,7 @@ class Installer(object):
             logging.warning(cmd)
             output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
             logging.warning(output)
-            os.remove('/tmp/mongodb.ip')
+            os.remove('/tmp/ferry/mongodb.ip')
 
         # Kill all the gunicorn instances. 
         logging.warning("stopping http servers")
@@ -573,14 +628,21 @@ class Installer(object):
     def _docker_running(self):
         return os.path.exists('/var/run/ferry.sock')
 
+    def _check_dockerimage(self, image, repo):
+        qualified = repo + '/' + image
+        cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' inspect ' + qualified + ' 2> /dev/null'
+        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        if output.strip() == '[]':
+            return image
+        else:
+            return None
+
     def _check_dockerfile(self, dockerfile, repo):
         not_installed = []
         images = self._get_image(dockerfile)
         for image in images:
-            qualified = DEFAULT_DOCKER_REPO + '/' + image
-            cmd = DOCKER_CMD + ' -H=' + DOCKER_SOCK + ' inspect ' + qualified + ' 2> /dev/null'
-            output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
-            if output.strip() == '[]':
+            i = self._check_dockerimage(image, DEFAULT_DOCKER_REPO)
+            if i:
                 not_installed.append(image)
         return not_installed
 
@@ -743,10 +805,10 @@ class Installer(object):
                     bflag = ' -s btrfs'
 
                 # Explicitly supply the DNS.
-                if options and '-n' in options:
+                if options and '-d' in options:
                     logging.warning("using custom dns")
                     dflag = ''
-                    for d in options['-n']:
+                    for d in options['-d']:
                         dflag += ' --dns %s' % d
                 else:
                     logging.warning("using public dns")
@@ -759,7 +821,7 @@ class Installer(object):
                 Popen(cmd, stdout=PIPE, shell=True)
 
                 # Wait a second to let the docker daemon do its thing.
-                time.sleep(2)
+                time.sleep(3)
                 return True, "Ferry daemon running on /var/run/ferry.sock"
             else:
                 return False, "Ferry appears to be already running. If this is an error, please type \'ferry clean\' and try again."
